% Demo of Hamming and rectangular windowing techniques
% Code adapted from http://mi.eng.cam.ac.uk/~ajr/SA95/node26.html

%% Set constants
n = 256; % epoch length
m = 128; % signal length
f = pi / 3; % define an arbitrary frequency

%% Create various windows
h = hamming(m); % create hamming window for a signal length m
h2 = hann(m)
h3 = barthannwin(m);
h4 = blackman(m);

%% Overlay plot of the window weights
figure; 
plot(h)
hold on; 
plot(h2,'-r')
plot(h3,'-k')
plot(h4,'-g')

% Put on pretty labels
text(100, 0.9, 'Hamming','Color', [0 0 1]);
text(100, 0.85, 'Hanning','Color', [1 0 0]);
text(100, 0.8, 'Bartlett-Hann','Color', [0 0 0]);
text(100, 0.75, 'Blackman','Color', [0 1 0]);

%% Pre-allocate arrays for windowed data
sr = zeros(size(1:n)); % fill entire viewing area with zeros for raw signal
sh = zeros(size(1:n)); % fill entire viewing area with zeros for hamming tapered signal
sh2 = zeros(size(1:n)); % fill entire viewing area with zeros for another tapered signal
sh3 = zeros(size(1:n)); % fill entire viewing area with zeros for another tapered signal
sh4 = zeros(size(1:n)); % fill entire viewing area with zeros for another tapered signal

%% Create raw signal and windowed signals
for i = 1:m ; sr(i) =        sin(f * i) ; end  % create sign wave of frequency f
for i = 1:m ; sh(i) = h(i) * sin(f * i) ; end  % apply Hamming window to sign wave of frequency f
for i = 1:m ; sh2(i) = h2(i) * sin(f * i) ; end  % apply Hanning window to sign wave of frequency f
for i = 1:m ; sh3(i) = h3(i) * sin(f * i) ; end  % apply Bart-Hann window to sign wave of frequency f
for i = 1:m ; sh4(i) = h4(i) * sin(f * i) ; end  % apply Blackman window to sign wave of frequency f

%% Get Power Spectra for Raw and Windows signals
ar=abs(fft(sr)); % FFT of raw signal
ah=abs(fft(sh)); % FFT of Hamming tapered signal
ah2=abs(fft(sh2)); % FFT of Hanning tapered signal
ah3=abs(fft(sh3)); % FFT of Bart-Hann tapered signal
ah4=abs(fft(sh4)); % FFT of Blackman tapered signal

%% Create plot of windowed signals and resultant spectra
figure;
subplot(5,2,1);
plot(sr,'-c'); % raw signal 
title('Raw');

subplot(5,2,3)
plot(sh,'-b') % Hamming tapered signal
title('Hamming');

subplot(5,2,5)
plot(sh2,'-r') % Hanning tapered signal
title('Hanning');

subplot(5,2,7)
plot(sh2,'-k') % Bartlett-Hann tapered signal
title('Bartlett');

subplot(5,2,9)
plot(sh2,'-g') % Blackman tapered signal
title('Blackman');

subplot(5,2,2)
plot(ar(1:n/2),'-c') % raw spectrum

subplot(5,2,4)
plot(ah(1:n/2),'-b') % Hamming tapered spectrum

subplot(5,2,6)
plot(ah2(1:n/2),'-r') % Hanning tapered spectrum

subplot(5,2,8)
plot(ah3(1:n/2),'-k') % Bartlett-Hann tapered spectrum

subplot(5,2,10)
plot(ah4(1:n/2),'-g') % Blackman tapered spectrum

%% Overlay Spectral results on one plot
figure;
plot(ar(1:n/2),'-c') % raw spectrum
hold on;
plot(ah(1:n/2),'-b') % Hamming tapered spectrum
plot(ah2(1:n/2),'-r') % Hanning tapered spectrum
plot(ah3(1:n/2),'-k') % Bartlett-Hann tapered spectrum
plot(ah4(1:n/2),'-g') % Blackman tapered spectrum

% Once again, pretty text
text(100, 60, 'Hamming','Color', [0 0 1]);
text(100, 55, 'Hanning','Color', [1 0 0]);
text(100, 50, 'Bartlett-Hann','Color', [0 0 0]);
text(100, 45, 'Blackman','Color', [0 1 0]);
